(* les variables en IMP *)
type var = string

type ar_exp = AVar of var | ACst of int | AAdd of ar_exp*ar_exp
  | AMul of ar_exp*ar_exp | AMin of ar_exp*ar_exp


type bool_exp =
  BTrue | BFalse | BAnd of bool_exp*bool_exp | BNeg of bool_exp
  | BEq of ar_exp*ar_exp  (* a1 = a2 *)
  | BLt of ar_exp*ar_exp  (* a1 < a2 *)

type command =
  CSkip | CAff of var*ar_exp | CSeq of command*command
  | CIf of bool_exp*command*command | CWhile of bool_exp*command


(*****************************************************************)
(* de simples fonctions d'affichage *)
let p = print_string
let pn = print_newline
let pi = print_int

let pr_var = p
let p_paren f =
  begin
    p "("; f(); p ")"
  end

let rec pr_ae = function
  | AVar v -> pr_var v
  | ACst k -> pi k
  | AAdd(a1,a2) -> p_paren (fun () -> pr_ae a1; p "+"; pr_ae a2)
  | AMul(a1,a2) -> p_paren (fun () -> pr_ae a1; p "*"; pr_ae a2)
  | AMin(a1,a2) -> p_paren (fun () -> pr_ae a1; p "-"; pr_ae a2)
    
let rec pr_be = function
  | BTrue -> p "true"
  | BFalse -> p "false"
  | BAnd(b1,b2) -> p_paren (fun () -> pr_be b1; p "&"; pr_be b2)
  | BNeg b -> begin p "not "; pr_be b end
  | BEq(a1,a2) -> p_paren (fun () -> pr_ae a1; p "="; pr_ae a2)
  | BLt(a1,a2) -> p_paren (fun () -> pr_ae a1; p "<"; pr_ae a2)

let is_seq = function | CSeq _ -> true | _ -> false
let blanc k = for i = 1 to k do p " " done

let rec pr_com k = function
  | CSkip -> blanc k; p "skip"
  | CAff(v,a) -> begin blanc k; pr_var v; p " <- "; pr_ae a end
  | CSeq(c1,c2) -> begin pr_com k c1; p ";"; pn(); pr_com k c2 end
  | CIf(b,c1,c2) ->
    begin
      p "if "; pr_be b; pn();
      p "then "; pn();
      pr_com (k+1) c1; pn();
      p "else "; pn();
      pr_com (k+1) c2
    end
  | CWhile(b,c) ->
    begin
      blanc k;
      p "while "; pr_be b;
      p " do ("; pn ();
      pr_com (k+1) c; pn ();
      blanc k;p ")"; 
    end

and pr_com_paren k c =
  if is_seq c then p_paren (fun () -> pr_com k c) else pr_com k c
